@@ -26,7 +26,7 @@ from tuxrun.requests import requests_get
 from tuxrun.results import Results
 from tuxrun.runtimes import Runtime
 from tuxrun.tests import Test
-from tuxrun.utils import ProgressIndicator
+from tuxrun.utils import get_new_output_dir, ProgressIndicator
 from tuxrun.writer import Writer
 from tuxrun.yaml import yaml_load
 
@@ -78,6 +78,7 @@ def run(options, tmpdir: Path) -> int:
         bl1=options.bl1,
         command=command,
         device=options.device,
+        qemu_image=options.qemu_image,
         dtb=options.dtb,
         kernel=options.kernel,
         ap_romfw=options.ap_romfw,
@@ -163,7 +164,7 @@ def run(options, tmpdir: Path) -> int:
     signal.alarm(job_timeout)
 
     # start the pre_run command
-    if options.device and options.device.flag_use_pre_run_cmd:
+    if options.device.flag_use_pre_run_cmd or options.qemu_image:
         LOG.debug("Pre run command")
         runtime.bind(tmpdir / "dispatcher" / "tmp", "/var/lib/lava/dispatcher/tmp")
         (tmpdir / "dispatcher" / "tmp").mkdir()
@@ -184,8 +185,13 @@ def run(options, tmpdir: Path) -> int:
     ]
 
     results = Results(options.tests)
-    # Start the writer (stderr or log-file)
-    with Writer(options.log_file) as writer:
+    # Start the writer (stdout or log-file)
+    with Writer(
+        options.log_file,
+        options.log_file_html,
+        options.log_file_text,
+        options.log_file_yaml,
+    ) as writer:
         # Start the runtime
         with runtime.run(args):
             for line in runtime.lines():
@@ -193,7 +199,10 @@ def run(options, tmpdir: Path) -> int:
                 results.parse(line)
     runtime.post_run()
     if options.results:
-        options.results.write_text(json.dumps(results.data))
+        if str(options.results) == "-":
+            sys.stdout.write(json.dumps(results.data) + "\n")
+        else:
+            options.results.write_text(json.dumps(results.data))
     return max([runtime.ret(), results.ret()])
 
 
@@ -223,6 +232,34 @@ def main() -> int:
                 if (tux.location / "dtbs" / "versatile-pb.dtb").exists():
                     options.dtb = tux.url + "/dtbs/versatile-pb.dtb"
 
+    cache_dir = None
+    if options.save_outputs:
+        if any(
+            (
+                o is None
+                for o in [
+                    options.log_file,
+                    options.log_file_html,
+                    options.log_file_text,
+                    options.log_file_yaml,
+                    options.results,
+                ]
+            )
+        ):
+            cache_dir = get_new_output_dir()
+            if options.log_file is None:
+                options.log_file = cache_dir / "logs"
+            if options.log_file_html is None:
+                options.log_file_html = cache_dir / "logs.html"
+            if options.log_file_text is None:
+                options.log_file_text = cache_dir / "logs.txt"
+            if options.log_file_yaml is None:
+                options.log_file_yaml = cache_dir / "logs.yaml"
+            if options.results is None:
+                options.results = cache_dir / "results.json"
+    elif options.log_file is None:
+        options.log_file = "-"
+
     if not options.device:
         parser.error("argument --device is required")
 
@@ -262,6 +299,8 @@ def main() -> int:
     finally:
         with contextlib.suppress(FileNotFoundError, PermissionError):
             shutil.rmtree(tmpdir)
+        if cache_dir:
+            print(f"TuxRun outputs saved to {cache_dir}")
 
 
 def start():
